#include <float.h>
#include "box.h"

Box::Box()
	: min(-0.5, -0.5, -0.5), max(0.5, 0.5, 0.5)
{
}

Box::Box(const Vector3 &min_arg, const Vector3 &max_arg)
	: min(min_arg), max(max_arg)
{
}

bool Box::contains(const Vector3 &pt) const
{
	return pt.x >= min.x && pt.y >= min.y && pt.z >= min.z &&
		pt.x < max.x && pt.y < max.y && pt.z < max.z;
}

bool Box::intersect(const Ray &ray, float *dist) const
{
	Vector3 param[2] = {min, max};
	Vector3 inv_dir(1.0 / ray.dir.x, 1.0 / ray.dir.y, 1.0 / ray.dir.z);
	int sign[3] = {inv_dir.x < 0, inv_dir.y < 0, inv_dir.z < 0};

	float tmin = (param[sign[0]].x - ray.origin.x) * inv_dir.x;
	float tmax = (param[1 - sign[0]].x - ray.origin.x) * inv_dir.x;
	float tymin = (param[sign[1]].y - ray.origin.y) * inv_dir.y;
	float tymax = (param[1 - sign[1]].y - ray.origin.y) * inv_dir.y;

	if(tmin > tymax || tymin > tmax) {
		return false;
	}
	if(tymin > tmin) {
		tmin = tymin;
	}
	if(tymax < tmax) {
		tmax = tymax;
	}

	float tzmin = (param[sign[2]].z - ray.origin.z) * inv_dir.z;
	float tzmax = (param[1 - sign[2]].z - ray.origin.z) * inv_dir.z;

	if(tmin > tzmax || tzmin > tmax) {
		return false;
	}
	if(tzmin > tmin) {
		tmin = tzmin;
	}
	if(tzmax < tmax) {
		tmax = tzmax;
	}

	float t = tmin < 1e-4 ? tmax : tmin;
	if(t >= 1e-4) {
		*dist = t;
		return true;
	}
	return false;
}
