#include "material.h"
#include "opengl.h"

Material::Material()
	: diffuse(1, 1, 1), specular(0, 0, 0), tex_scale(1, 1)
{
	alpha = 1.0;
	shininess = 60.0;

	memset(tex, 0, sizeof tex);
}

void Material::bind(bool use_tex) const
{
	float col[4];

	col[0] = diffuse.x;
	col[1] = diffuse.y;
	col[2] = diffuse.z;
	col[3] = alpha;
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, col);

	col[0] = specular.x;
	col[1] = specular.y;
	col[2] = specular.z;
	col[3] = 1.0;
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, col);

	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

	if(!use_tex) {
		return;
	}

	glActiveTexture(GL_TEXTURE0);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glScalef(tex_scale.x, tex_scale.y, 1);

	for(int i=0; i<MAX_MTL_TEXTURES; i++) {
		if(tex[i]) {
			bind_texture(tex[i], i);
		} else {
			bind_texture(0, i);
		}
	}
	glActiveTexture(GL_TEXTURE0);

	glMatrixMode(GL_MODELVIEW);
}
