#ifndef TERRAIN_H_
#define TERRAIN_H_

#include <vector>
#include "vmath/vmath.h"
#include "material.h"

class Terrain {
private:
	float dx, dy;	// inter-pixel distance in uv-space
	int xsz, ysz;
	float *hmap;

	float scale, height_scale;

	std::vector<Material> layers;
	unsigned int sdr;

	mutable int dlist, dlist_norm;
	mutable int dlist_sub[2], dlist_norm_sub[2];
	mutable float dlist_norm_sz;

	void invalidate_mesh();

public:
	Terrain();
	~Terrain();

	int add_layer();
	int add_layer(const Material &mtl);
	int add_layer(Texture *tex, int textype = TEX_DIFFUSE);
	bool set_layer_texture(int idx, Texture *tex, int textype = TEX_DIFFUSE);

	const Material &get_layer(int idx) const;
	Material &get_layer(int idx);

	void set_shader(unsigned int sdr);
	unsigned int get_shader() const;

	void set_scale(float s);
	float get_scale() const;

	void set_height_scale(float s);
	float get_height_scale() const;

	bool load(const char *fname);
	int get_map_width() const { return xsz; }
	int get_map_height() const { return ysz; }

	Vector2 world_to_uv(const Vector2 &pt) const;
	Vector2 uv_to_world(const Vector2 &pt) const;

	float lookup(int x, int y) const;

	float get_height(const Vector2 &pt) const;
	Vector3 get_normal(const Vector2 &pt) const;

	float get_world_height(const Vector2 &pt) const;

	bool intersect(const Ray &ray, float *dist) const;
	bool intersect_micro(const Ray &ray, float *dist, float thres = 0.01) const;

	void draw(int usub = 0, int vsub = 0) const;

	void draw_mesh(int usub = 0, int vsub = 0) const;
	void draw_normals(float sz = 1.0, int usub = 0, int vsub = 0) const;
};

#endif	// TERRAIN_H_
